#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


def get_allteams():
   all_teams = pd.DataFrame()

   for t in teams:
      print t
      df = htmlToPandas('html/{team}.html'.format(team=t), t)
      all_teams = pd.concat([all_teams, df])

   all_teams.to_pickle('nba_teams.pkl')


def get_dataframe(filename, table_id):
   with open(filename, 'r') as fd:
      soup = BeautifulSoup(fd.read())

   try:
      table = soup.find('table', {'id': table_id})
      body = table.find('tbody')
      rows = body.find_all('tr', {'class': ''})
   except AttributeError:
      print "Parseing {0} failed".format(filename)
      return df.DataFrame()

   rows = [str(r.encode('utf-8')) for r in rows if r['class'] == ['']]

   html = '<table>' + ''.join(rows) + '</table>'

   df = pd.io.html.read_html(html, infer_types=False)[0]

   return df


def get_pergame():

   df = pd.DataFrame()

   for t in teams:
      for y in years:
         fname = "html/{0}_{1}.html".format(t, y)
         tmp = get_dataframe(fname, 'per_game')
         tmp.columns = cols
         tmp['year'] = int(y)
         df = df.append(tmp)

   df['Age'] = df['Age'].astype(int)
   df['G'] = df['G'].astype(int)
   df['GS'] = df['GS'].astype(int)
   df['MP'] = df['MP'].astype(float)
   df['FG'] = df['FG'].astype(float)
   df['FGA'] = df['FGA'].astype(float)
   df['FG%'] = df['FG%'].astype(float)
   df['3P'] = df['3P'].astype(float)
   df['3PA'] = df['3PA'].astype(float)
   df['3P%'] = df['3P%'].astype(float)
   df['2P'] = df['2P'].astype(float)
   df['2PA'] = df['2PA'].astype(float)
   df['2P%'] = df['2P%'].astype(float)
   df['FT'] = df['FT'].astype(float)
   df['FTA'] = df['FTA'].astype(float)
   df['FT%'] = df['FT%'].astype(float)
   df['ORB'] = df['ORB'].astype(float)
   df['DRB'] = df['DRB'].astype(float)
   df['TRB'] = df['TRB'].astype(float)
   df['AST'] = df['AST'].astype(float)
   df['STL'] = df['STL'].astype(float)
   df['BLK'] = df['BLK'].astype(float)
   df['TOV'] = df['TOV'].astype(float)
   df['PF'] = df['PF'].astype(float)
   df['PTS'] = df['PTS'].astype(float)

   return df


def htmlToPandas(filename, name):

   cols = ['Season', 'Lg', 'Team', 'W', 'L', 'W/L%', 'Finish', 'SRS', 'Pace',
           'Rel_Pace', 'ORtg', 'Rel_ORtg', 'DRtg', 'Rel_DRtg', 'Playoffs',
           'Coaches', 'WS']

   df = get_dataframe(filename, name)

   df.columns = cols

   df['WS'].replace(r'\xc2\xa0',
                    value=' ',
                    inplace=True,
                    regex=True)
   df['Team'] = name
   df['Season'].replace(r'-\d\d$',
                        value='',
                        inplace=True,
                        regex=True)
   df['Season'] = df['Season'].astype(int)
   df['W'] = df['W'].astype(int)
   df['L'] = df['L'].astype(int)
   df['W/L%'] = df['W/L%'].astype(float)
   df['Finish'] = df['Finish'].astype(float)
   df['SRS'] = df['SRS'].astype(float)
   df['Pace'] = df['Pace'].astype(float)
   df['Rel_Pace'] = df['Rel_Pace'].astype(float)
   df['ORtg'] = df['ORtg'].astype(float)
   df['Rel_ORtg'] = df['Rel_ORtg'].astype(float)
   df['DRtg'] = df['DRtg'].astype(float)
   df['Rel_DRtg'] = df['Rel_DRtg'].astype(float)

   with open('tmp.html', 'w') as fd:
      fd.write(df.to_html().encode('utf-8'))

   return df


def create_pages(df):

   pages = []
   pages.append({'title': 'Value Data',
                 'obj': data,
                 'fantasyID': 'value',
                 'href': 'value-data.html',
                 'cols': ['rank',
                          'Player',
                          'Pos',
                          'FTeam',
                          'Tm',
                          'G',
                          'GS',
                          'MP',
                          'FG%',
                          'FT%',
                          'TRB',
                          'AST',
                          'STL',
                          'BLK',
                          'PTS',
                          'NormalizedValue',
                          'NaiveValue']})
   pages.append({'title': 'Value by Position',
                 'obj': positional,
                 'fantasyID': 'normalized-value-positional',
                 'href': 'normalized-value-positional.html',
                 'cols': ['Player',
                          'Pos',
                          'FTeam',
                          'Tm',
                          'G',
                          'GS',
                          'MP',
                          'FG%',
                          'FT%',
                          'TRB',
                          'AST',
                          'STL',
                          'BLK',
                          'PTS',
                          'NormalizedValue',
                          'NaiveValue']})
   pages.append({'title': 'Positional Breakdown By Mean',
                 'obj': pbMean,
                 'fantasyID': 'pb-mean',
                 'href': 'pb-mean.html',
                 'cols': ['G', 'GS',
                          'MP',
                          'FG%',
                          'FT%',
                          'TRB',
                          'AST',
                          'STL',
                          'BLK',
                          'PTS',
                          'NormalizedValue',
                          'NaiveValue']})
   pages.append({'title': 'Team Average Value',
                 'obj': teamValue,
                 'fantasyID': 'teamValue',
                 'href': 'teamValue.html',
                 'cols': ['G',
                          'GS',
                          'MP',
                          'FG%',
                          'FT%',
                          'TRB',
                          'AST',
                          'STL',
                          'BLK',
                          'PTS',
                          'NormalizedValue',
                          'NaiveValue']})
   pages.append({'title': 'Rosters',
                 'obj': rosters,
                 'fantasyID': 'rosters',
                 'href': 'rosters.html',
                 'cols': ['rank',
                          'Player',
                          'Pos',
                          'FTeam',
                          'Tm',
                          'G',
                          'GS',
                          'MP',
                          'FG%',
                          'FT%',
                          'TRB',
                          'AST',
                          'STL',
                          'BLK',
                          'PTS',
                          'NormalizedValue',
                          'NaiveValue']})


def process_pages(pages):
   j2_env = Environment(loader=FileSystemLoader('templates'), trim_blocks=True)
   baseTemplate = j2_env.get_template('fantasy-template.html')
   tocTemplate = j2_env.get_template('toc.html')
   posTemplate = j2_env.get_template('positional-template.html')
   chartsTemplate = j2_env.get_template('charts-template.html')

   expr1 = re.compile(r"<tr>.*rank.*</thead>", re.MULTILINE | re.DOTALL)
   expr2 = re.compile(r"<tr>.*Pos.*</thead>", re.MULTILINE | re.DOTALL)
   expr3 = re.compile(r"<tr>.*FTeam.*</thead>", re.MULTILINE | re.DOTALL)

   for p in pages:
      fantasyID = p['fantasyID']
      htmlText = p['obj'].to_html(columns=p['cols'],
                                  classes=["table", "table-bordered"])

      if fantasyID == 'pb-mean':
         expr = expr2
      elif fantasyID == 'teamValue':
         expr = expr3
      else:
         expr = expr1

      tmp = ('normalized-value-positional' == fantasyID) or \
      ('rosters' == fantasyID)

      if tmp:
         for k in htmlText.keys():
            newKey = re.sub(r'\s', '_', k)
            newKey = re.sub(r'\+', '_', newKey)
            htmlText.rename({k: newKey}, inplace=True)

         for k in htmlText.keys():
               txt = htmlText[k][:7]
               htmlText[k] = txt
               htmlText[k] += 'id="{0}" '.format(k)
               htmlText[k] += txt
               htmlText[k] = re.sub(expr, "</thead>", htmlText[k])
         template = posTemplate
      else:
         htmlText = htmlText[:7] + 'id="{0}" '.format(fantasyID) + htmlText[7:]
         htmlText = re.sub(expr, "</thead>", htmlText)
         template = baseTemplate

      with open(os.path.join(self.filesPath, p['href']), 'w') as fd:
         text = template.render(title=p['title'],
                                fantasy_table=htmlText,
                                fantasy_id=fantasyID,
                                allPages=pages)
         fd.write(text)

      with open(os.path.join(self.filesPath, 'charts.html'), 'w') as fd:
         text = chartsTemplate.render(title='Charts',
                                      figs=self.figures,
                                      allPages=pages)
         fd.write(text)

      with open(os.path.join(self.filesPath, 'toc.html'), 'w') as fd:
         text = tocTemplate.render(title='Table of Contents',
                                   pages=pages,
                                   chartsUrl='charts.html',
                                   allPages=pages)
         fd.write(text)



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pycurl
import pandas as pd
from bs4 import BeautifulSoup


class ESPN(object):

   def __init__(self, year, leagueID):
      """
         init ESPN object

         :param year: The year that league takes place
         :param leagueID: The ESPN league ID
      """

      self.processLeague()

   def processLeague(self):
      """
         extract info from standings and league info, place into dataframe
      """

      self.teams = {}
      soup = BeautifulSoup(self.standingsBuf.getvalue())
      tables = soup.findAll("table")

      for table in tables:
         if table.findAll(text='EAST'):
            east = table
         if table.findAll(text='WEST'):
            west = table

      teams = east.findAll('tr')[2:]
      for t in teams:
         cols = t.findAll('td')
         d = {}
         d['wins'] = int(cols[1].text)
         d['losses'] = int(cols[2].text)
         d['ties'] = int(cols[3].text)
         d['conf'] = 'east'
         teamName = (cols[0].text).lower()
         self.teams[teamName] = d

      teams = west.findAll('tr')[2:]
      for t in teams:
         cols = t.findAll('td')
         d = {}
         d['wins'] = int(cols[1].text)
         d['losses'] = int(cols[2].text)
         d['ties'] = int(cols[3].text)
         d['conf'] = 'west'
         teamName = (cols[0].text).lower()
         self.teams[teamName] = d

      soup = BeautifulSoup(self.leagueBuf.getvalue())
      tables = soup.findAll("table", attrs={"class": "playerTableTable"})
      for table in tables:
         teamName = table.findAll('tr')[0].findAll('a')[0].text
         teamName = teamName.lower()
         rows = table.findAll('tr')[2:]
         players = []
         for row in rows:
            try:
               player = row.findAll('a')[0].text
               players.append(player)
            except:
               pass

         self.teams[teamName]['players'] = players



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
from bs4 import BeautifulSoup
import pandas as pd
import errno

from Dataframe_Augmenter import augment_minutes
from Dataframe_Augmenter import augment_price
from Dataframe_Augmenter import augment_value
from Util import mkdir_p

teams = [u'SAS', u'OKC', u'CHI', u'BOS', u'PHO', u'MEM', u'ORL', u'NYK',
         u'PHI', u'NOH', u'UTA', u'ATL', u'DEN', u'IND', u'HOU', u'SAC',
         u'CHA', u'LAL', u'DET', u'BRK', u'MIN', u'GSW', u'TOR', u'POR',
         u'WAS', u'LAC', u'MIA', u'MIL', u'CLE', u'DAL', u'NOP']


def get_player_stats(data_dir, year):
   d = os.path.join(data_dir, 'raw_data', 'teams', str(year))
   pkl = os.path.join(data_dir, 'processed_data', str(year))
   df = get_players(d, year)
   df = augment_minutes(df)
   df = augment_value(df)
   df = augment_price(df)
   mkdir_p(pkl)
   pkl = os.path.join(pkl, 'team_data.pkl')
   df.to_pickle(pkl)


def get_dataframe(filename, table_id):
   if not os.path.isfile(filename):
      print "Cannot open file, try downloading data\n{0}".format(filename)
      sys.exit(1)

   with open(filename, 'r') as fd:
      soup = BeautifulSoup(fd.read())

   try:
      table = soup.find('table', {'id': table_id})
      body = table.find('tbody')
      rows = body.find_all('tr', {'class': ''})
   except AttributeError:
      print "Parsing {0} failed".format(filename)
      return df.DataFrame()

   rows = [str(r.encode('utf-8')) for r in rows if r['class'] == ['']]

   html = '<table>' + ''.join(rows) + '</table>'

   df = pd.io.html.read_html(html, infer_types=False)[0]

   return df


def get_advanced(data_dir, year):

   df = pd.DataFrame()

   cols = ['Rk', 'Player', 'Age', 'G', 'MP', 'PER', 'TS%', 'eFG%', 'FTr',
           '3PAr', 'ORB%', 'DRB%', 'TRB%', 'AST%', 'STL%', 'BLK%', 'TOV%',
           'USG%', 'ORtg', 'DRtg', 'OWS', 'DWS', 'WS', 'WS/48']

   for t in teams:
      filename = os.path.join(data_dir, "{0}.html".format(t))
      if os.path.isfile(filename):
         tmp = get_dataframe(filename, 'advanced')
         tmp.columns = cols
         tmp['year'] = int(year)
         df = df.append(tmp)

   del df['Rk']
   del df['Age']
   del df['G']
   df['MP'] = df['MP'].astype(int)
   df['PER'] = df['PER'].astype(float)
   df['TS%'] = df['TS%'].astype(float)
   df['eFG%'] = df['eFG%'].astype(float)
   df['FTr'] = df['FTr'].astype(float)
   df['3PAr'] = df['3PAr'].astype(float)
   df['ORB%'] = df['ORB%'].astype(float)
   df['DRB%'] = df['DRB%'].astype(float)
   df['TRB%'] = df['TRB%'].astype(float)
   df['AST%'] = df['AST%'].astype(float)
   df['STL%'] = df['STL%'].astype(float)
   df['BLK%'] = df['BLK%'].astype(float)
   df['TOV%'] = df['TOV%'].astype(float)
   df['USG%'] = df['USG%'].astype(float)
   df['ORtg'] = df['ORtg'].astype(float)
   df['DRtg'] = df['DRtg'].astype(float)
   df['OWS'] = df['OWS'].astype(float)
   df['DWS'] = df['DWS'].astype(float)
   df['WS'] = df['WS'].astype(float)
   df['WS/48'] = df['WS/48'].astype(float)

   return df


def get_pergame(data_dir, year):

   df = pd.DataFrame()

   cols = ['ind', 'Player', 'Age', 'G', 'GS', 'MP', 'FG', 'FGA', 'FG%', '3P',
           '3PA', '3P%', '2P', '2PA', '2P%', 'FT', 'FTA', 'FT%', 'ORB', 'DRB',
           'TRB', 'AST', 'STL', 'BLK', 'TOV', 'PF', 'PTS']

   for t in teams:
      filename = os.path.join(data_dir, "{0}.html".format(t))
      if os.path.isfile(filename):
         tmp = get_dataframe(filename, 'per_game')
         tmp.columns = cols
         tmp['year'] = int(year)
         df = df.append(tmp)

   del df['MP']
   df['Age'] = df['Age'].astype(int)
   df['G'] = df['G'].astype(int)
   df['GS'] = df['GS'].astype(int)
   df['FG'] = df['FG'].astype(float)
   df['FGA'] = df['FGA'].astype(float)
   df['FG%'] = df['FG%'].astype(float)
   df['3P'] = df['3P'].astype(float)
   df['3PA'] = df['3PA'].astype(float)
   df['3P%'] = df['3P%'].astype(float)
   df['2P'] = df['2P'].astype(float)
   df['2PA'] = df['2PA'].astype(float)
   df['2P%'] = df['2P%'].astype(float)
   df['FT'] = df['FT'].astype(float)
   df['FTA'] = df['FTA'].astype(float)
   df['FT%'] = df['FT%'].astype(float)
   df['ORB'] = df['ORB'].astype(float)
   df['DRB'] = df['DRB'].astype(float)
   df['TRB'] = df['TRB'].astype(float)
   df['AST'] = df['AST'].astype(float)
   df['STL'] = df['STL'].astype(float)
   df['BLK'] = df['BLK'].astype(float)
   df['TOV'] = df['TOV'].astype(float)
   df['PF'] = df['PF'].astype(float)
   df['PTS'] = df['PTS'].astype(float)
   del df['ind']

   return df


def get_salaries(data_dir, year):
   df = pd.DataFrame()

   cols = ['ind', 'Player', 'Salary']

   for t in teams:
      filename = os.path.join(data_dir, "{0}.html".format(t))
      if os.path.isfile(filename):
         tmp = get_dataframe(filename, 'salaries')
         tmp.columns = cols
         tmp['year'] = int(year)
         df = df.append(tmp, ignore_index=True)

   df['Salary'] = df['Salary'].str.replace(r'[$,]', '').astype('float')
   del df['ind']

   return df


def get_roster(data_dir, year):
   df = pd.DataFrame()

   cols = ['No.', 'Player', 'Pos', 'Ht', 'Wt',
           'Birth Date', 'Experience', 'College']

   none_opened = True
   for t in teams:
      filename = os.path.join(data_dir, "{0}.html".format(t))
      if os.path.isfile(filename):
         tmp = get_dataframe(filename, 'roster')
         tmp.columns = cols
         tmp['year'] = year
         df = df.append(tmp)
         none_opened = False

   if none_opened:
      print "Could not find raw data in {0}".format(data_dir)
      sys.exit(1)

   del df['No.']

   # replace positions so that only C-PF-SF-SG-PG exist
   replacement = {"Pos": {'PF-SF': 'PF', 'PG-SG': 'PG', 'PG-SG': 'PG',
                          'SF-PF': 'SF', 'SF-SG': 'SF', '^G$': 'SG',
                          'G-F': 'SG', 'F': 'PF', 'G-PF': 'G'}}
   df.replace(to_replace=replacement, inplace=True)

   # set height to be in inches for all players
   def tmp(s):
      t = s.split('-')
      return int(t[0]) * 12 + int(t[1])

   df['Experience'].replace('R', 0, inplace=True)
   df['Ht'] = df['Ht'].apply(tmp)

   df['Wt'] = df['Wt'].astype(int)

   return df


def get_players(data_dir, year):
   df1 = get_roster(data_dir, year)
   df2 = get_pergame(data_dir, year)
   df3 = get_salaries(data_dir, year)
   df4 = get_advanced(data_dir, year)
   del df2['year']
   del df3['year']
   del df4['year']

   # FIXME -- players who get traded wind up as dupes.  here I just drop
   # drop their first team, which isn't optimal, their stats should be
   # averaged or something.
   df1.drop_duplicates('Player', inplace=True, take_last=True)
   df2.drop_duplicates('Player', inplace=True, take_last=True)
   df3.drop_duplicates('Player', inplace=True, take_last=True)
   df4.drop_duplicates('Player', inplace=True, take_last=True)
   df5 = pd.merge(df1, df2, left_on="Player", right_on="Player", how="outer")
   df6 = pd.merge(df5, df3, left_on="Player", right_on="Player", how="outer")
   df7 = pd.merge(df6, df4, left_on="Player", right_on="Player", how="outer")

   return df7


def htmlToPandas(filename, name):

   cols = ['Season', 'Lg', 'Team', 'W', 'L', 'W/L%', 'Finish', 'SRS', 'Pace',
           'Rel_Pace', 'ORtg', 'Rel_ORtg', 'DRtg', 'Rel_DRtg', 'Playoffs',
           'Coaches', 'WS']

   df = get_dataframe(filename, name)

   df.columns = cols

   df['WS'].replace(r'\xc2\xa0',
                    value=' ',
                    inplace=True,
                    regex=True)
   df['Team'] = name
   df['Season'].replace(r'-\d\d$',
                        value='',
                        inplace=True,
                        regex=True)
   df['Season'] = df['Season'].astype(int)
   df['W'] = df['W'].astype(int)
   df['L'] = df['L'].astype(int)
   df['W/L%'] = df['W/L%'].astype(float)
   df['Finish'] = df['Finish'].astype(float)
   df['SRS'] = df['SRS'].astype(float)
   df['Pace'] = df['Pace'].astype(float)
   df['Rel_Pace'] = df['Rel_Pace'].astype(float)
   df['ORtg'] = df['ORtg'].astype(float)
   df['Rel_ORtg'] = df['Rel_ORtg'].astype(float)
   df['DRtg'] = df['DRtg'].astype(float)
   df['Rel_DRtg'] = df['Rel_DRtg'].astype(float)

   with open('tmp.html', 'w') as fd:
      fd.write(df.to_html().encode('utf-8'))

   return df



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import copy
import cStringIO
import re
import os
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from bs4 import BeautifulSoup
from jinja2 import Environment, FileSystemLoader

pd.set_option('display.max_columns', None)
pd.set_option('display.float_format', None)
pd.set_option('display.precision', 4)
htmlRoot = "html/"
minimumMinutesPlayed = 200

data_dir = 'html_data'


class RawStats(object):

   def __init__(self):
      self.url = "www.basketball-reference.com/leagues/NBA_2014_totals.html"
      self.csvName = 'nba_stats.csv'
      self.columns = ['', 'Rk', 'Player', 'Pos', 'Age', 'Tm', 'G', 'GS',
                      'MP', 'FG', 'FGA', 'FG%', '3P', '3PA', '3P%', '2P',
                      '2PA', '2P%', 'FT', 'FTA', 'FT%', 'ORB', 'DRB',
                      'TRB', 'AST', 'STL', 'BLK', 'TOV', 'PF', 'PTS', 'FTeam']

      if not os.path.isfile(self.csvName):
         self.downloadHTML()
         self.processHTML()
         self.writeCSV()
      else:
         self.data = pd.read_csv(self.csvName)
         self.data.columns = self.columns

      self.data.fillna(0.0)
      replacement = {"Pos": {'PF-SF': 'PF', 'PG-SG': 'PG', 'PG-SG': 'PG',
                     'SF-PF': 'SF', 'SF-SG': 'SF', '^G$': 'SG'}}
      self.data.replace(to_replace=replacement, inplace=True, regex=True)

      return

   def writeCSV(self):
      self.data.to_csv(self.csvName)

   def downloadHTML(self):
      self.downloadDrafts()
      self.downloadDrafts()

   def downloadDrafts(self):

      years = range(1950, 2014)
      years.reverse()

      for y in years:
         downloadDraft(y)
         time.sleep(10.0)

   def downloadTeams(self):

      teams = [u'SAS', u'OKC', u'CHI', u'BOS', u'PHO', u'MEM', u'ORL', u'NYK',
               u'PHI', u'NOH', u'UTA', u'ATL', u'DEN', u'IND', u'HOU', u'SAC',
               u'CHA', u'LAL', u'DET', u'BRK', u'MIN', u'GSW', u'TOR', u'POR',
               u'WAS', u'LAC', u'MIA', u'MIL', u'CLE', u'DAL']

      for y in ['2013', '2014', '2012']:
         for t in teams:
            print "downloading {0}, {1}".format(t, y)
            self.downloadTeam(t, y)
            time.sleep(10.0)

   def processHTML(self):
      """ Finds the table in the downloaded HTML and converts it into a CSV """

      soup = BeautifulSoup(self.buf.getvalue())
      table = soup.find("table")

      thead = table.findAll("thead")[0]
      tbody = table.findAll("tbody")[0]

      labels = [th.getText() for th in thead.findAll("th")]
      rows = tbody.findAll("tr")

      self.stats = pd.DataFrame(columns=labels)

      tmpStats = []

      for row in rows:
         if row.find('th') is not None:
            continue
         vals = [td.getText() for td in row.findAll("td")]
         # if no statistic, fill with 0
         vals = [0 if w == '' else w for w in vals]
         tmpStats.append(dict(zip(labels, vals)))

      self.data = pd.DataFrame(columns=labels, data=tmpStats, dtype=float)
      self.data = self.data[self.data.MP > minimumMinutesPlayed]
      self.data[u'FTeam'] = 'FA'

      ############
      all_teams = pd.DataFrame()

      for t in teams:
         print t
         df = htmlToPandas('html/{team}.html'.format(team=t), t)
         all_teams = pd.concat([all_teams, df])

      ###############
         with open(filename, 'r') as fd:
            soup = BeautifulSoup(fd.read())

         try:
            table = soup.find('table', {'id': table_id})
            body = table.find('tbody')
            rows = body.find_all('tr', {'class': ''})
         except AttributeError:
            print "Parseing {0} failed".format(filename)
            return df.DataFrame()

         rows = [str(r.encode('utf-8')) for r in rows if r['class'] == ['']]

         html = '<table>' + ''.join(rows) + '</table>'

         df = pd.io.html.read_html(html, infer_types=False)[0]

         return df

      ####################

         df = pd.DataFrame()

         for t in teams:
            for y in years:
               fname = "html/{0}_{1}.html".format(t, y)
               tmp = get_dataframe(fname, 'per_game')
               tmp.columns = cols
               tmp['year'] = int(y)
               df = df.append(tmp)

         df['Age'] = df['Age'].astype(int)
         df['G'] = df['G'].astype(int)
         df['GS'] = df['GS'].astype(int)
         df['MP'] = df['MP'].astype(float)
         df['FG'] = df['FG'].astype(float)
         df['FGA'] = df['FGA'].astype(float)
         df['FG%'] = df['FG%'].astype(float)
         df['3P'] = df['3P'].astype(float)
         df['3PA'] = df['3PA'].astype(float)
         df['3P%'] = df['3P%'].astype(float)
         df['2P'] = df['2P'].astype(float)
         df['2PA'] = df['2PA'].astype(float)
         df['2P%'] = df['2P%'].astype(float)
         df['FT'] = df['FT'].astype(float)
         df['FTA'] = df['FTA'].astype(float)
         df['FT%'] = df['FT%'].astype(float)
         df['ORB'] = df['ORB'].astype(float)
         df['DRB'] = df['DRB'].astype(float)
         df['TRB'] = df['TRB'].astype(float)
         df['AST'] = df['AST'].astype(float)
         df['STL'] = df['STL'].astype(float)
         df['BLK'] = df['BLK'].astype(float)
         df['TOV'] = df['TOV'].astype(float)
         df['PF'] = df['PF'].astype(float)
         df['PTS'] = df['PTS'].astype(float)

         return df



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Devin Kelly"

import pycurl
import time
import sys
import os
import errno

from Util import mkdir_p

base_draft_url = "http://www.basketball-reference.com/draft/NBA_{year}.html"
base_team_url = "http://www.basketball-reference.com/teams/{team}/{year}.html"
default_dir = os.path.expanduser("~/.fantasy_basketball")


def download_data(data_dir, teams, drafts, league, year, league_id):
   data_dir = os.path.join(data_dir, 'raw_data')
   if teams:
      download_teams(data_dir, year)

   if drafts:
      download_draft(data_dir, year)

   if (league_id) and (league_id is not None):
      download_league(data_dir, leagueID, year)


def download_drafts(years):

   years = range(1950, 2014)
   years.reverse()

   for y in years:
      download_draft(y)
      time.sleep(10.0)


def download_teams(data_dir, year):

   teams = [u'SAS', u'OKC', u'CHI', u'BOS', u'PHO', u'MEM', u'ORL', u'NYK',
            u'PHI', u'NOH', u'UTA', u'ATL', u'DEN', u'IND', u'HOU', u'SAC',
            u'CHA', u'LAL', u'DET', u'BRK', u'MIN', u'GSW', u'TOR', u'POR',
            u'WAS', u'LAC', u'MIA', u'MIL', u'CLE', u'DAL']

   for t in teams:
      print "downloading {0}, {1}".format(t, year)
      download_team(data_dir, t, year)
      time.sleep(10.0)


def download_draft(data_dir, year):

   data_dir = os.path.join(data_dir, "draft", str(year))
   mkdir_p(data_dir)

   filename = "draft.html"
   filename = os.path.join(data_dir, filename)

   fp = open(filename, "wb")
   curl = pycurl.Curl()
   curl.setopt(pycurl.URL, base_draft_url.format(year=year))
   curl.setopt(pycurl.FOLLOWLOCATION, 1)
   curl.setopt(pycurl.MAXREDIRS, 5)
   curl.setopt(pycurl.CONNECTTIMEOUT, 30)
   curl.setopt(pycurl.TIMEOUT, 300)
   curl.setopt(pycurl.NOSIGNAL, 1)
   curl.setopt(pycurl.WRITEDATA, fp)
   try:
      curl.perform()
   except:
      import traceback
      traceback.print_exc(file=sys.stderr)
   curl.close()
   fp.close()

   return


def download_team(data_dir, team, year=time.strftime('%Y', time.localtime())):

   data_dir = os.path.join(data_dir, "teams", str(year))
   mkdir_p(data_dir)

   filename = "{team}.html".format(team=team)
   filename = os.path.join(data_dir, filename)

   fp = open(filename, "wb")
   curl = pycurl.Curl()
   curl.setopt(pycurl.URL, base_team_url.format(team=team, year=year))
   curl.setopt(pycurl.FOLLOWLOCATION, 1)
   curl.setopt(pycurl.MAXREDIRS, 5)
   curl.setopt(pycurl.CONNECTTIMEOUT, 30)
   curl.setopt(pycurl.TIMEOUT, 300)
   curl.setopt(pycurl.NOSIGNAL, 1)
   curl.setopt(pycurl.WRITEDATA, fp)
   try:
      curl.perform()
   except:
      import traceback
      traceback.print_exc(file=sys.stderr)
   curl.close()
   fp.close()

   return


def download_league(data_dir, leagueID, year):
   """
      fetch standings and team data from espn.com
   """
   leagueURL = "http://games.espn.go.com/fba/leaguerosters?" +\
               "leagueId={0}&seasonID={1}".format(leagueID, year)
   standingsURL = "http://games.espn.go.com/fba/standings?" +\
                  "leagueId={0}&seasonID={1}".format(leagueID, year)

   data_dir = os.path.join(data_dir, "league", str(year))
   mkdir_p(data_dir)
   league_filename = "league.html"
   league_filename = os.path.join(data_dir, filename)
   standings_filename = "standings.html"
   standings_filename = os.path.join(data_dir, filename)

   league_fd = open(league_filename, "w")
   standings_fd = open(standings_filename, "w")

   c = pycurl.Curl()
   c.setopt(c.URL, leagueURL)
   c.setopt(c.WRITEFUNCTION, league_fd)

   try:
      c.perform()
   except:
      import traceback
      traceback.print_exc(file=sys.stderr)

   c.setopt(c.URL, standingsURL)
   c.setopt(c.WRITEFUNCTION, standings_fd)

   try:
      c.perform()
   except:
      import traceback
      traceback.print_exc(file=sys.stderr)

   c.close()
   league_fd.close()
   standings_fd.close()



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Devin Kelly"

import copy
import pycurl
import cStringIO
import re
import os
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from bs4 import BeautifulSoup
from jinja2 import Environment, FileSystemLoader

pd.set_option('display.max_columns', None)
pd.set_option('display.float_format', None)
pd.set_option('display.precision', 4)
htmlRoot = "html/"
minimumMinutesPlayed = 200

data_dir = 'html_data'


def htmlToPandas(filename, name):

   cols = ['Season', 'Lg', 'Team', 'W', 'L', 'W/L%', 'Finish', 'SRS', 'Pace',
           'Rel_Pace', 'ORtg', 'Rel_ORtg', 'DRtg', 'Rel_DRtg', 'Playoffs',
           'Coaches', 'WS']

   df = get_dataframe(filename, name)

   df.columns = cols

   df['WS'].replace(r'\xc2\xa0',
                    value=' ',
                    inplace=True,
                    regex=True)
   df['Team'] = name
   df['Season'].replace(r'-\d\d$',
                        value='',
                        inplace=True,
                        regex=True)
   df['Season'] = df['Season'].astype(int)
   df['W'] = df['W'].astype(int)
   df['L'] = df['L'].astype(int)
   df['W/L%'] = df['W/L%'].astype(float)
   df['Finish'] = df['Finish'].astype(float)
   df['SRS'] = df['SRS'].astype(float)
   df['Pace'] = df['Pace'].astype(float)
   df['Rel_Pace'] = df['Rel_Pace'].astype(float)
   df['ORtg'] = df['ORtg'].astype(float)
   df['Rel_ORtg'] = df['Rel_ORtg'].astype(float)
   df['DRtg'] = df['DRtg'].astype(float)
   df['Rel_DRtg'] = df['Rel_DRtg'].astype(float)

   with open('tmp.html', 'w') as fd:
      fd.write(df.to_html().encode('utf-8'))

   return df



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
from bs4 import BeautifulSoup
import pandas as pd
import errno
import numpy as np

from Util import mkdir_p

import matplotlib.pyplot as plt
import re


class Plotter(object):

   def __init__(self, data_dir, year):
      default_processed_data_dir = os.path.join(data_dir, 'processed_data')
      default_plot_dir = os.path.join(data_dir, 'plots')
      self.save_dir = os.path.join(default_plot_dir, year)
      fname = os.path.join(default_processed_data_dir, year, 'team_data.pkl')

      self.df = pd.read_pickle(fname)
      self.year = year

      self.make_positional_df()

      mkdir_p(self.save_dir)

   def make_positional_df(self):
      self.C = self.df[self.df.Pos == 'C']
      self.PF = self.df[self.df.Pos == 'PF']
      self.SF = self.df[self.df.Pos == 'SF']
      self.SG = self.df[self.df.Pos == 'SG']
      self.PG = self.df[self.df.Pos == 'PG']

   def is_plot_func(self, s):
      if re.search("^plot_", s) is not None:
         attr = getattr(self, s)
         if callable(attr):
            return True

      return False

   def make_all_plots(self, img_format='eps'):

      attrs = dir(self)

      plot_funcs = [x for x in attrs if self.is_plot_func(x)]

      for func in plot_funcs:
         f = getattr(self, func)
         f(img_format)

   def plot_value_hist(self, img_format="eps"):

      self.df.hist('value', bins=20)

      fig = plt.gcf()
      ax = plt.gca()
      ax.set_ylabel('Frequency')
      ax.set_xlabel('Value')
      ax.set_title('Value Histogram')
      fname = "value_histogram.{0}".format(img_format)
      fig.savefig(os.path.join(self.save_dir, fname))

   def plot_stats_hist(self, img_format="eps"):

      stats = ['FG%', 'FT%', 'TRB', 'AST', 'BLK', 'PTS']

      for stat in stats:
         self.df.hist(stat, bins=20)
         fig = plt.gcf()
         ax = plt.gca()
         ax.set_xlabel(stat)
         ax.set_ylabel('Frequency')
         ax.set_title('{0} Histogram'.format(stat))
         fname = "{0}_histogram.{1}".format(stat, img_format)
         fig.savefig(os.path.join(self.save_dir, fname))

   def plot_value_hist_by_pos(self, img_format="eps"):

      for pos in ['C', 'PF', 'SF', 'SG', 'PG']:
         df = getattr(self, pos)
         df.hist('value', bins=20)
         fig = plt.gcf()
         ax = plt.gca()
         ax.set_xlabel('Value')
         ax.set_ylabel('Frequency')
         ax.set_title('Value Histogram for {0}'.format(pos))
         fname = "value_histogram_{0}.{1}".format(pos, img_format)
         fig.savefig(os.path.join(self.save_dir, fname))

   def plot_value_by_pos(self, img_format="eps"):

      C_avg_value = self.C.mean()['value']
      PF_avg_value = self.PF.mean()['value']
      SF_avg_value = self.SF.mean()['value']
      SG_avg_value = self.SG.mean()['value']
      PG_avg_value = self.PG.mean()['value']

      y = (C_avg_value, PF_avg_value, SF_avg_value, SG_avg_value, PG_avg_value)

      N = 5
      width = 0.5

      ind = np.arange(N)  # the x locations for the groups

      fig, ax = plt.subplots()
      rects1 = ax.bar(ind, y)

      # add some text for labels, title and axes ticks
      ax.set_ylabel('Average Value')
      ax.set_title('Average Value by Position')
      ax.set_xticks(ind + width)
      ax.set_xticklabels(('C', 'PF', 'SF', 'SG', 'PG'))

      fname = "value_by_pos.".format(img_format)
      fig.savefig(os.path.join(self.save_dir, fname))

   def plot_top_50_by_pos(self, img_format="eps"):
      top_50 = self.df.sort('value', ascending=False)[0:50]

      C = sum(top_50.Pos == 'C')
      PF = sum(top_50.Pos == 'PF')
      SF = sum(top_50.Pos == 'SF')
      SG = sum(top_50.Pos == 'SG')
      PG = sum(top_50.Pos == 'PG')

      y = [C, PF, SF, SG, PG]

      N = 5
      width = 0.5

      ind = np.arange(N)  # the x locations for the groups

      fig, ax = plt.subplots()
      rects1 = ax.bar(ind, y)

      # add some text for labels, title and axes ticks
      ax.set_ylabel('Number of Players')
      ax.set_xlabel('Position')
      ax.set_title('Number of players in the top 50 value, by position')
      ax.set_xticks(ind + width)
      ax.set_xticklabels(('C', 'PF', 'SF', 'SG', 'PG'))

      fname = "top_50_value_by_pos.{0}".format(img_format)
      fig.savefig(os.path.join(self.save_dir, fname))



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pandas as pd
import re
from bs4 import BeautifulSoup
from jinja2 import Environment, FileSystemLoader

__author__ = "Devin Kelly"

teams = [u'SAS', u'OKC', u'CHI', u'BOS', u'PHO', u'MEM', u'ORL', u'NYK',
         u'PHI', u'NOH', u'UTA', u'ATL', u'DEN', u'IND', u'HOU', u'SAC',
         u'CHA', u'LAL', u'DET', u'BRK', u'MIN', u'GSW', u'TOR', u'POR',
         u'WAS', u'LAC', u'MIA', u'MIL', u'CLE', u'DAL']

years = ['2012', '2013', '2014']


def augment_minutes(df, minutes=400):
   df = df[df.MP > minutes]
   return df


def cleanup(df):
   df = df.fillna(0.0)
   df = df.dropna()
   df = augment_minutes(df)
   return df


def augment_value(df):

   df['value'] = (df['FG%'] - df['FG%'].mean()) / df['FG%'].std() + \
                 (df['FT%'] - df['FG%'].mean()) / df['FT%'].std() + \
                 (df['TRB'] - df['TRB'].mean()) / df['TRB'].std() + \
                 (df['AST'] - df['AST'].mean()) / df['AST'].std() + \
                 (df['BLK'] - df['BLK'].mean()) / df['BLK'].std() + \
                 (df['PTS'] - df['PTS'].mean()) / df['PTS'].std()

   return df


def augment_price(df, nplayers=8, money_per_player=200, players_per_team=11):

   total_picks = nplayers * players_per_team
   money_supply = float(nplayers * money_per_player)

   df['price'] = 0.0
   years = list(set(df['year']))
   for y in years:
      top_players = df[df.year == y]
      top_players = top_players.sort('value', ascending=False)[0:total_picks]
      total_value = top_players['value'].sum()

      for ii in top_players.Player:
         player_value = top_players[top_players.Player == ii].value
         player_price = money_supply * (player_value / total_value)
         df.price[(df.year == y) & (df.Player == ii)] = player_price

   return df



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
from Download import download_data
from Process import get_player_stats
from Plot import Plotter
from Util import mkdir_p


default_dir = os.path.expanduser("~/.fantasy_basketball")
default_raw_data_dir = os.path.join(default_dir, "raw_data")
default_processed_data_dir = os.path.join(default_dir, "processed_data")
default_html_dir = os.path.join(default_dir, "html")
default_plot_dir = os.path.join(default_dir, "plot")



#!/home/devin/software/my_projects/fantasy_basketball/test_env/bin/python2
#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = "Devin Kelly"

import os
import time
import click
import errno

from Fantasy_Basketball import download_data
from Fantasy_Basketball import get_player_stats
from Fantasy_Basketball import default_dir
from Fantasy_Basketball import default_raw_data_dir
from Fantasy_Basketball import default_processed_data_dir
from Fantasy_Basketball import default_plot_dir
from Fantasy_Basketball import default_html_dir
from Fantasy_Basketball import mkdir_p
from Fantasy_Basketball import Plotter


@click.group()
def cli():
   pass


@cli.command()
@click.option('--data_dir',
              default=default_dir,
              help='Download Fantasy Basketball Data')
@click.option('--teams', is_flag=True, default=False,
              help="Download NBA Team Data Only")
@click.option('--draft', is_flag=True, default=False,
              help="Download Draft Data Only")
@click.option('--league', is_flag=True, default=False,
              help="Download Fantasy League Data Only")
@click.option('--year', default=time.strftime('%Y', time.localtime()),
              help="The year to use downloading stats")
@click.option('--league_id', default=None,
              help="The ESPN League ID to use downloading stats")
def download(data_dir, teams, draft, league, year, league_id):
   click.echo('Downloading to {0}'.format(data_dir))
   download_data(data_dir, teams, draft, league, year, league_id)


@cli.command()
@click.option('--data_dir',
              default=default_dir,
              help='Process Fantasy Basketball Data')
@click.option('--teams', is_flag=True, default=False,
              help="Process NBA Team Data Only")
@click.option('--draft', is_flag=True, default=False,
              help="Process Draft Data Only")
@click.option('--league', is_flag=True, default=False,
              help="Process Fantasy League Data Only")
@click.option('--year', default=time.strftime('%Y', time.localtime()),
              help="The year to use downloading stats")
def process(data_dir, teams, draft, league, year):
   click.echo('Processing to {0}'.format(data_dir))
   get_player_stats(data_dir, year)


@cli.command()
@click.option('--data_dir',
              default=default_dir,
              help='Process Fantasy Basketball Data')
@click.option('--teams', is_flag=True, default=False,
              help="Write HTML for NBA Team Data Only")
@click.option('--draft', is_flag=True, default=False,
              help="Write HTML for Draft Data Only")
@click.option('--league', is_flag=True, default=False,
              help="Write HTML for Fantasy League Data Only")
def write_html(data_dir):
   click.echo('Writing HTML Data to {0}'.format(data_dir))


@cli.command()
@click.option('--data_dir',
              default=default_dir,
              help='Process Fantasy Basketball Data')
@click.option('--year', default=time.strftime('%Y', time.localtime()),
              help="The year to use downloading stats")
def plot(data_dir, year):
   click.echo('Plotting to {0}'.format(data_dir))
   plotter = Plotter(data_dir, year)
   plotter.make_all_plots()


def main():
   cli()

   return

if __name__ == "__main__":
   main()



#  Copyright (C) 2014 Devin Kelly
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from bs4 import BeautifulSoup
from jinja2 import Environment, FileSystemLoader

def gen_web(data_dir, df):
   pages = []
   p.append(get_value_page(df))

def get_value_page(df):
   p = {'title': 'Value Data',
        'obj': df,
        'fantasyID': 'value',
        'href': 'value-data.html',
        'cols': ['rank', 'Player', 'Pos', 'FTeam', 'Tm', 'G',
        'GS', 'MP', 'FG%', 'FT%', 'TRB', 'AST', 'STL',
        'BLK', 'PTS', 'NormalizedValue', 'NaiveValue']}
   return p


def process_pages(df):
   j2_env = Environment(loader=FileSystemLoader('templates'), trim_blocks=True)
   baseTemplate = j2_env.get_template('fantasy-template.html')
   tocTemplate = j2_env.get_template('toc.html')
   posTemplate = j2_env.get_template('positional-template.html')
   chartsTemplate = j2_env.get_template('charts-template.html')

   expr1 = re.compile(r"<tr>.*rank.*</thead>", re.MULTILINE | re.DOTALL)
   expr2 = re.compile(r"<tr>.*Pos.*</thead>", re.MULTILINE | re.DOTALL)
   expr3 = re.compile(r"<tr>.*FTeam.*</thead>", re.MULTILINE | re.DOTALL)

   for p in pages:
      fantasyID = p['fantasyID']
      htmlText = p['obj'].to_html(columns=p['cols'],
                                  classes=["table", "table-bordered"])

      if fantasyID == 'pb-mean':
         expr = expr2
      elif fantasyID == 'teamValue':
         expr = expr3
      else:
         expr = expr1

      tmp = ('normalized-value-positional' == fantasyID) or \
      ('rosters' == fantasyID)

      if tmp:
         for k in htmlText.keys():
            newKey = re.sub(r'\s', '_', k)
            newKey = re.sub(r'\+', '_', newKey)
            htmlText.rename({k: newKey}, inplace=True)

         for k in htmlText.keys():
               txt = htmlText[k][:7]
               htmlText[k] = txt
               htmlText[k] += 'id="{0}" '.format(k)
               htmlText[k] += txt
               htmlText[k] = re.sub(expr, "</thead>", htmlText[k])
         template = posTemplate
      else:
         htmlText = htmlText[:7] + 'id="{0}" '.format(fantasyID) + htmlText[7:]
         htmlText = re.sub(expr, "</thead>", htmlText)
         template = baseTemplate

      with open(os.path.join(self.filesPath, p['href']), 'w') as fd:
         text = template.render(title=p['title'],
                                fantasy_table=htmlText,
                                fantasy_id=fantasyID,
                                allPages=pages)
         fd.write(text)

      with open(os.path.join(self.filesPath, 'charts.html'), 'w') as fd:
         text = chartsTemplate.render(title='Charts',
                                      figs=self.figures,
                                      allPages=pages)
         fd.write(text)

      with open(os.path.join(self.filesPath, 'toc.html'), 'w') as fd:
         text = tocTemplate.render(title='Table of Contents',
                                   pages=pages,
                                   chartsUrl='charts.html',
                                   allPages=pages)
         fd.write(text)
